import sys
from argparse import ArgumentParser

from loguru import logger

from . import BaseAutoNLPCommand


def upload_command_factory(args):
    return UploadCommand(args.project, args.split, args.col_mapping, args.files)


class UploadCommand(BaseAutoNLPCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        upload_parser = parser.add_parser("upload")
        upload_parser.add_argument("--project", type=str, default=None, required=True, help="Project Name")
        upload_parser.add_argument(
            "--split", type=str, default=None, required=True, help="File Split Type", choices=["train", "valid"]
        )
        upload_parser.add_argument(
            "--col_mapping", type=str, default=None, required=True, help="Column Mapping. E.g. col1:map1,col2:map2"
        )
        upload_parser.add_argument(
            "--files", type=str, required=True, help="Paths to the files to upload, comma-separated"
        )
        upload_parser.set_defaults(func=upload_command_factory)

    def __init__(self, name: str, split: str, col_mapping: str, files: str):
        self._name = name
        self._split = split
        self._col_mapping = col_mapping
        self._files = files

    def run(self):
        from ..autonlp import AutoNLP

        logger.info(f"Uploading files for project: {self._name}")
        client = AutoNLP()
        try:
            project = client.get_project(name=self._name)
        except ValueError:
            logger.error(f"Project {self._name} not found! You can create it using the create_project command.")
            sys.exit(1)
        splits = self._col_mapping.split(",")
        col_maps = {}
        for s in splits:
            k, v = s.split(":")
            col_maps[k] = v
        logger.info(f"Mapping: {col_maps}")

        files = self._files.split(",")
        project.upload(filepaths=files, split=self._split, col_mapping=col_maps)



from argparse import ArgumentParser

from loguru import logger

from ..languages import SUPPORTED_LANGUAGES
from ..tasks import TASKS
from . import BaseAutoNLPCommand


def create_project_command_factory(args):
    return CreateProjectCommand(args.name, args.task, args.language)


class CreateProjectCommand(BaseAutoNLPCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        create_project_parser = parser.add_parser("create_project")
        create_project_parser.add_argument("--name", type=str, default=None, required=True, help="Project Name")
        create_project_parser.add_argument(
            "--task", type=str, default=None, required=True, help="Project Task Type", choices=list(TASKS.keys())
        )
        create_project_parser.add_argument(
            "--language", type=str, default=None, required=True, help="Language", choices=SUPPORTED_LANGUAGES
        )
        create_project_parser.set_defaults(func=create_project_command_factory)

    def __init__(self, name: str, task: str, language: str):
        self._name = name
        self._task = task
        self._lang = language

    def run(self):
        from ..autonlp import AutoNLP

        logger.info(f"Creating project: {self._name} with task: {self._task}")
        client = AutoNLP()
        project = client.create_project(name=self._name, task=self._task, language=self._lang)
        print(project)



from argparse import ArgumentParser

from . import BaseAutoNLPCommand


def login_command_factory(args):
    return LoginCommand(api_key=args.api_key)


class LoginCommand(BaseAutoNLPCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        login_parser = parser.add_parser("login")
        login_parser.add_argument("--api-key", type=str, required=True, help="API key")
        login_parser.set_defaults(func=login_command_factory)

    def __init__(self, api_key: str):
        self._api_key = api_key

    def run(self):
        from ..autonlp import AutoNLP

        client = AutoNLP()
        client.login(token=self._api_key)



import sys
from argparse import ArgumentParser

from loguru import logger

from . import BaseAutoNLPCommand


def create_project_command_factory(args):
    return ProjectInfoCommand(args.name)


class ProjectInfoCommand(BaseAutoNLPCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        create_project_parser = parser.add_parser("project_info")
        create_project_parser.add_argument("--name", type=str, default=None, required=True, help="Project Name")
        create_project_parser.set_defaults(func=create_project_command_factory)

    def __init__(self, name: str):
        self._name = name

    def run(self):
        from ..autonlp import AutoNLP

        logger.info(f"Fetching info for project: {self._name}")
        client = AutoNLP()
        try:
            project = client.get_project(name=self._name)
        except ValueError:
            logger.error(f"Project {self._name} not found! You can create it using the create_project command.")
            sys.exit(1)
        print(project)



from dataclasses import dataclass

import requests
from prettytable import PrettyTable

from .utils import BOLD_TAG, RESET_TAG, http_get


@dataclass
class Metrics:
    _token: str
    username: str
    project_name: str
    project_id: int
    language: str

    @classmethod
    def from_json_resp(cls, json_resp: dict, token: str, project_name: str, username: str):
        return cls(
            project_id=json_resp["id"],
            language=json_resp["config"]["language"],
            _token=token,
            project_name=project_name,
            username=username,
        )

    def model_metrics(self, model_id):
        json_resp = http_get(f"/models/{self.username}/{model_id}", token=self._token, suppress_logs=True).json()
        eval_dicts = [ed for ed in json_resp if "eval_loss" in ed]
        eval_dicts = sorted(eval_dicts, key=lambda k: k["eval_loss"])
        if len(eval_dicts) > 0:
            return eval_dicts[0]

    def print(self):
        resp = http_get(path=f"/projects/{self.project_id}/jobs", token=self._token, suppress_logs=True)
        json_jobs = resp.json()
        printout = ["~" * 35, f"🚀 AutoNLP Model Metrics for project: {self.project_name}", "~" * 35, ""]
        printout.append(f"💾 you have {len(json_jobs)} models in this project")
        print("\n".join(printout))
        print("")
        best_loss = 99999999
        best_model = None
        print_logs = PrettyTable([f"📚 {BOLD_TAG}Model ID{RESET_TAG}", f"📊 {BOLD_TAG}Eval Loss{RESET_TAG}"])
        for job in json_jobs:
            model_id = job["id"]
            try:
                metrics = self.model_metrics(model_id=model_id)
                eval_loss = metrics["eval_loss"]
                if eval_loss < best_loss:
                    best_loss = eval_loss
                    best_model = model_id
                print_logs.add_row([model_id, eval_loss])
            except (requests.exceptions.HTTPError, TypeError):
                continue
        print(print_logs)
        print("")
        print(f"🎖 Best Model: {best_model}, Best Loss: {best_loss}")



from dataclasses import dataclass
from typing import Dict, List, Optional, Union

from prettytable import PrettyTable


@dataclass
class Model:
    _token: str
    json_resp: Union[List, List[Dict[str, float]]]
    username: str
    model_id: int
    max_train_print_count: Optional[int] = 7

    @classmethod
    def from_json_resp(cls, json_resp: Union[List, List[Dict[str, float]]], token: str, username: str, model_id: str):
        return cls(_token=token, json_resp=json_resp, username=username, model_id=model_id)

    def print(self):
        printout = ["~" * 35, f"🚀 AutoNLP Model Info for model_id # {self.model_id}", "~" * 35, ""]
        if len(self.json_resp) == 0:
            printout.append("🚨 Model ID found but has no entries yet!!!")
            print("\n".join(printout))

        training_log = [log for log in self.json_resp if "loss" in log]
        valid_log = [log for log in self.json_resp if "eval_loss" in log]

        printout.append("⭐️ Training Log:")
        print_training_logs = PrettyTable(["Epoch", "Loss"])
        # train_losses = [log["loss"] for log in training_log]
        for log in training_log:
            print_training_logs.add_row([log["epoch"], log["loss"]])

        print("\n".join(printout))
        print(print_training_logs)
        printout = []

        printout.append("")
        printout.append("~" * 35)
        printout.append("")

        print("\n".join(printout))
        printout = []

        printout.append("⭐️ Validation Log:")
        print_validation_logs = PrettyTable(["Epoch", "Loss"])
        for log in valid_log:
            print_validation_logs.add_row([log["epoch"], log["eval_loss"]])

        print("\n".join(printout))
        print(print_validation_logs)



"""
Copyright 2020 The HuggingFace Team
"""

import json
import os

import requests
from loguru import logger

from . import config
from .languages import SUPPORTED_LANGUAGES
from .metrics import Metrics
from .model import Model
from .project import Project
from .tasks import TASKS
from .utils import UnauthenticatedError, http_get, http_post


class AutoNLP:
    def __init__(self, config_dir: str = None) -> None:
        self.username = None
        self.token = None
        self._project = None
        self.config_dir = config_dir
        if self.config_dir is None:
            home_dir = os.path.expanduser("~")
            self.config_dir = os.path.join(home_dir, ".autonlp")
        os.makedirs(self.config_dir, exist_ok=True)

    def get_token(self):
        """Retrieve API token, or raise UnauthenticatedError"""
        self._login_from_conf()
        if self.token is None:
            raise UnauthenticatedError("❌ Credentials not found ! Please login to AutoNLP first.")
        return self.token

    def login(self, token: str):
        """Login to AutoNLP"""
        if token.startswith("api_org"):
            logger.error("⚠ Authenticating as an organization is not allowed. Please provide a user API key.")
            raise ValueError("Login with an organization API keys are not supported")
        try:
            auth_resp = http_get(path="/whoami-v2", domain=config.HF_API, token=token, token_prefix="Bearer")
        except requests.HTTPError as err:
            if err.response.status_code == 401:
                logger.error("❌ Failed to authenticate. Check the passed token is valid!")
            raise
        user_identity = auth_resp.json()
        self.username = user_identity["name"]
        logger.info(f"🗝 Successfully logged in as {self.username}")
        orgs = []
        if user_identity["type"] == "user":
            orgs = [org["name"] for org in user_identity["orgs"]]
        self.orgs = orgs
        self.token = token
        login_dict = {"username": self.username, "orgs": self.orgs, "token": token}
        logger.info(f"🗝 Storing credentials in:  {self.config_dir}")
        with open(os.path.join(self.config_dir, "autonlp.json"), "w") as fp:
            json.dump(login_dict, fp)

    def _login_from_conf(self):
        """Retrieve credentials from local config"""
        conf_json = None
        if self.username is None or self.token is None:
            logger.info("🗝 Retrieving credentials from config...")
            if os.path.isfile(os.path.join(self.config_dir, "autonlp.json")):
                with open(os.path.join(self.config_dir, "autonlp.json"), "r") as conf_file:
                    conf_json = json.load(conf_file)
                    if conf_json is None:
                        raise UnauthenticatedError("❌ Credentials not found! Please login to AutoNLP first.")
                    else:
                        self.username = conf_json["username"]
                        self.orgs = conf_json["orgs"]
                        self.token = conf_json["token"]

    def create_project(self, name: str, task: str, language: str):
        """Create a project and return it"""
        self._login_from_conf()
        task_id = TASKS.get(task)
        language = str(language).strip().lower()
        if len(language) != 2 or language not in SUPPORTED_LANGUAGES:
            raise ValueError(f"❌ Invalid language selected. Please choose one of {SUPPORTED_LANGUAGES}")
        if task_id is None:
            raise ValueError(f"❌ Invalid task specified. Please choose one of {list(TASKS.keys())}")
        payload = {
            "username": self.username,
            "proj_name": name,
            "task": task_id,
            "config": {"version": 0, "patch": 1, "language": language},
        }
        json_resp = http_post(path="/projects/create", payload=payload, token=self.token).json()
        proj_name = json_resp["proj_name"]
        if json_resp["created"] is True:
            logger.info(f"✅ Successfully created project: '{proj_name}'!")
        else:
            logger.info(f"🤙 Project '{proj_name}' already exists, it was loaded successfully.")
        self._project = Project.from_json_resp(json_resp, token=self.token)
        self._project.refresh()
        return self._project

    def get_project(self, name):
        """Retrieves a project"""
        self._login_from_conf()
        if self.username is None:
            raise UnauthenticatedError("❌ Credentials not found ! Please login to AutoNLP first.")
        if self._project is None or self._project.name != name:
            logger.info(f"☁ Retrieving project '{name}' from AutoNLP...")
            try:
                json_resp = http_get(path=f"/projects/{self.username}/{name}", token=self.token).json()
            except requests.exceptions.HTTPError as err:
                if err.response.status_code == 404:
                    raise ValueError(f"❌ Project '{name}' not found. Please create the project using create_project")
                else:
                    raise
            self._project = Project.from_json_resp(json_resp, token=self.token)
            self._project.refresh()
        else:
            self._project.refresh()
        logger.info(f"✅ Successfully loaded project: '{name}'!")
        return self._project

    def get_metrics(self, model_id, project):
        self._login_from_conf()
        if self.username is None:
            raise UnauthenticatedError("❌ Credentials not found ! Please login to AutoNLP first.")
        if model_id is not None:
            try:
                json_resp = http_get(f"/models/{self.username}/{model_id}", token=self.token).json()
            except requests.exceptions.HTTPError as err:
                if err.response.status_code == 404:
                    raise ValueError(f"❌ Model '{model_id}' not found.") from err
                raise
            _model_info = Model.from_json_resp(
                json_resp=json_resp, token=self.token, username=self.username, model_id=model_id
            )
            return _model_info.print()
        if project is not None:
            try:
                json_resp = http_get(path=f"/projects/{self.username}/{project}", token=self.token).json()
            except requests.exceptions.HTTPError as err:
                if err.response.status_code == 404:
                    raise ValueError(f"❌ Project '{project}' not found!") from err
                raise
            _metrics = Metrics.from_json_resp(
                json_resp=json_resp, token=self.token, project_name=project, username=self.username
            )
            return _metrics.print()

    def predict(self, model_id, input_text):
        self._login_from_conf()
        if self.username is None:
            raise UnauthenticatedError("❌ Credentials not found ! Please login to AutoNLP first.")
        try:
            payload = {"username": self.username, "model_id": model_id, "input_text": input_text}
            json_resp = http_post(path="/models/predict", payload=payload, token=self.token).json()
            return json_resp
        except requests.exceptions.HTTPError as err:
            if err.response.status_code == 404:
                raise ValueError(f"❌ Model '{model_id}' not found.") from err
            raise


if __name__ == "__main__":
    client = AutoNLP()
    client.login(token="TEST_KEY")
    project = client.create_project(name="imdb_test_4", task="binary_classification")
    token = client.get_token()
    # project = client.get_project(name="imdb_test_4")
    col_mapping = {"sentiment": "target", "review": "text"}

    train_files = ["/home/abhishek/datasets/imdb_folds.csv"]
    valid_files = ["/home/abhishek/datasets/imdb_valid.csv"]
    project.upload(train_files, split="train", col_mapping=col_mapping)
    project.upload(valid_files, split="valid", col_mapping=col_mapping)
    project.train()
    project.refresh()
    print(project)



# Lint as: python3
""" HuggingFace/AutoNLP
"""

from setuptools import find_packages, setup


DOCLINES = __doc__.split("\n")

INSTALL_REQUIRES = ["loguru==0.5.3", "requests==2.25.1", "tqdm==4.56.0", "prettytable==2.0.0"]

QUALITY_REQUIRE = [
    "black",
    "isort",
    "flake8==3.7.9",
]


EXTRAS_REQUIRE = {
    "dev": INSTALL_REQUIRES + QUALITY_REQUIRE,
    "quality": INSTALL_REQUIRES + QUALITY_REQUIRE,
    "docs": [
        "recommonmark",
        "sphinx==3.1.2",
        "sphinx-markdown-tables",
        "sphinx-rtd-theme==0.4.3",
        "sphinx-copybutton",
    ],
}

setup(
    name="autonlp",
    version="0.0.3",
    description=DOCLINES[0],
    long_description="\n".join(DOCLINES[2:]),
    author="HuggingFace Inc.",
    author_email="abhishek@huggingface.co",
    url="https://github.com/huggingface/autonlp",
    download_url="https://github.com/huggingface/autonlp/tags",
    license="Apache 2.0",
    package_dir={"": "src"},
    packages=find_packages("src"),
    scripts=["scripts/autonlp"],
    extras_require=EXTRAS_REQUIRE,
    install_requires=INSTALL_REQUIRES,
    entry_points={"console_scripts": ["autonlp=autonlp.cli.autonlp:main"]},
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    keywords="automl autonlp huggingface",
)



import sys
from argparse import ArgumentParser

from loguru import logger

from . import BaseAutoNLPCommand


def train_command_factory(args):
    return TrainCommand(args.project)


class TrainCommand(BaseAutoNLPCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        train_parser = parser.add_parser("train")
        train_parser.add_argument("--project", type=str, default=None, required=True, help="Project Name")
        train_parser.set_defaults(func=train_command_factory)

    def __init__(self, name: str):
        self._name = name

    def run(self):
        from ..autonlp import AutoNLP

        logger.info(f"Starting Training For Project: {self._name}")

        client = AutoNLP()
        try:
            project = client.get_project(name=self._name)
        except ValueError:
            logger.error(f"Project {self._name} not found! You can create it using the create_project command.")
            sys.exit(1)
        project.train()
